<?php
/*
Plugin Name: phpLivePacker
Plugin URI: 
Description: Integrates the "clear phpLivePacker cache" option into the WordPress admin interface. More information about phpLivePacker and the documentation can be found on phplivepacker.bartlweb.net.
Version: 1.1
Author: Christian Bartl (bartlweb)
Author URI: http://phplivepacker.bartlweb.net/
*/

# **********************************************************************************************
# bw_phpLivePacker
#
# Copyright 2013 Christian Bartl (phplivepacker.bartlweb.net)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# This notice must appear in all copies of the script.
# **********************************************************************************************



/* ------------------------------------------------------------------------------------------------------------
   ajax request
------------------------------------------------------------------------------------------------------------ */

add_action('wp_ajax_bw_phplivepacker_clearcache', 'bw_phplivepacker_clearcache');

function bw_phplivepacker_clearcache() {
	if(clearCacheDirectory()) {
		if(incrementConstant()) {
			die("phpLivepacker cache successfully cleared.");
		} else {
			die("phpLivepacker cache cleared, but version constant could not be incremented.\n\nPlease check phpLivePacker plugin settings and filesystem permissions.");
		}
	} else {
		die("phpLivepacker cache could not be cleared.\n\nPlease check phpLivePacker plugin settings and filesystem permissions.");
	}
	
	
}


/* ------------------------------------------------------------------------------------------------------------
   clear phplivepacker cache directory
------------------------------------------------------------------------------------------------------------ */

function clearCacheDirectory() {
	// check if directory is configured
	if(!get_option('bw_phplivepacker_cachedirectory')) {
		return false;
	}
	
	// generate full path
	$path = trim(trim(get_option('bw_phplivepacker_cachedirectory')), '/');
	$path = ABSPATH . $path  . '/';
	
	if(preg_match('/^Windows/', $_ENV['OS']) || $_SERVER['WINDIR']) {
		$path = str_replace('/', '\\', $path);
	} else {
		$path = str_replace('\\', '/', $path);
	}

	// check on path traversal
	if(strpos('..', $path)) {
		return false;
	}
	
	// check if directory exists
	if(!is_dir($path)) {
		return false;
	}
	
	// check if directory is writeable
	if(!is_writeable($path)) {
		return false;
	}

	// delete all files except .htaccess and .htpasswd
	foreach(glob($path.'*') as $file){
		if(!preg_match('/\.htaccess/', $file) && !preg_match('/\.htpasswd$/', $file)) {
			unlink($file);
		}
	}
	
	
	return true;
}


/* ------------------------------------------------------------------------------------------------------------
   increment phplivepacker constant
------------------------------------------------------------------------------------------------------------ */
	
function incrementConstant() {

	// check if version increment is enabled
	if(!get_option('bw_phplivepacker_enableversionincrement')) {
		return true;
	}

	// check if file is configured
	if(!get_option('bw_phplivepacker_phpfile')) {
		return false;
	}
	
	// generate full path
	$path = trim(trim(get_option('bw_phplivepacker_phpfile')), '/');
	$path = ABSPATH . $path;
	
	if(preg_match('/^Windows/', $_ENV['OS']) || $_SERVER['WINDIR']) {
		$path = str_replace('/', '\\', $path);
	} else {
		$path = str_replace('\\', '/', $path);
	}

	// check on path traversal
	if(strpos('..', $path)) {
		return false;
	}

	// check if file exists
	if(!is_file($path)) {
		return false;
	}

	// check if file is readable and writeable
	if(!is_readable($path) || !is_writeable($path)) {
		return false;
	}

	// parse file and increment constant
	$content = file_get_contents($path);
	$content_lines = explode("\n", $content);

	foreach($content_lines as $i => $line) {
		if(preg_match('/^\s*define\(\s*["\\\']' . get_option('bw_phplivepacker_versionconstant'). '/', $line)) {
			$tmp = explode(",", $line);
			$line = $tmp[0] . ", '" . (intval(str_replace(array('"', "'"), '', trim($tmp[1])))+1) . "');";
			$content_lines[$i] = $line;
		}
	}
	
	file_put_contents($path, implode("\n", $content_lines));
	
	
	return true;
}


/* ------------------------------------------------------------------------------------------------------------
   add clear cache button to backend
------------------------------------------------------------------------------------------------------------ */

if(is_admin()){
	add_action('wp_before_admin_bar_render', 'bw_phplivepacker_admin_bar_clearCacheButton');
} 

function bw_phplivepacker_admin_bar_clearCacheButton() {
	global $wp_admin_bar;
	
	// only show for admins
	if (!is_user_logged_in()){
		return false;
	}
	if (!is_super_admin() || !is_admin_bar_showing()) {
		return false;
	}

	// generate js for ajax request
	$javascript = array();	
	$javascript[] = 'jQuery.post(';
	$javascript[] = 'ajaxurl,';
	$javascript[] = '{action:"bw_phplivepacker_clearcache"},';
	$javascript[] = 'function(data){alert(data);}';
	$javascript[] = ');';
	$javascript[] = 'return false;';
	
	// generate button
	$wp_admin_bar->add_menu(array(
		'parent' => false,
		'id' => 'bw_phplivepacker_clearcache',
		'title' => __('<span style="background: url('.plugins_url('bw_phplivepacker').'/res/be_icon.png) no-repeat center left; padding-left: 22px;">Clear phpLivePacker cache</span>'),
		'href' => '#',
		'meta' => array(
			'onclick' => implode("", $javascript),
			'title' => "click to clear phpLivePacker cache"
		)
	));
}


/* ------------------------------------------------------------------------------------------------------------
   create options pages
------------------------------------------------------------------------------------------------------------ */

if(is_admin()){
	add_action('admin_menu', 'bw_phplivepacker_admin_add_page');
	add_action('admin_init', 'bw_phplivepacker_register_settings');
} 


// add new settings page to menu
function bw_phplivepacker_admin_add_page() {
	add_options_page('phpLivePacker Settings', 'phpLivePacker', 'manage_options', 'bwphplivepacker-settingspage', 'bw_phplivepacker_options_page');
}


// register and configure settings sections and fields
function bw_phplivepacker_register_settings() {
	// register setting options
	register_setting('bw_phplivepacker', 'bw_phplivepacker_cachedirectory');
	register_setting('bw_phplivepacker', 'bw_phplivepacker_enableversionincrement');
	register_setting('bw_phplivepacker', 'bw_phplivepacker_phpfile');
	register_setting('bw_phplivepacker', 'bw_phplivepacker_versionconstant');
	
	// define sections to group setting options
	add_settings_section(
	    'bwphplivepacker-defaultsection',
	    'Setting',
	    'bw_phplivepacker_sectionInfo_defaultsection',
	    'bwphplivepacker-settingspage'
	);
	
	// define fields for setting options
	add_settings_field(
	    'bw_phplivepacker_cachedirectory', 
	    'phpLivePacker cache directory', 
	    'bw_phplivepacker_createOptionsField_cacheDirectory', 
	    'bwphplivepacker-settingspage',
	    'bwphplivepacker-defaultsection'			
	);
	add_settings_field(
	    'bw_phplivepacker_enableversionincrement', 
	    'enable autoincrement of version constant', 
	    'bw_phplivepacker_createOptionsField_enableVersionIncrement', 
	    'bwphplivepacker-settingspage',
	    'bwphplivepacker-defaultsection'			
	);
	add_settings_field(
	    'bw_phplivepacker_phpfile', 
	    'phpLivePacker PHP file', 
	    'bw_phplivepacker_createOptionsField_phpFile', 
	    'bwphplivepacker-settingspage',
	    'bwphplivepacker-defaultsection'			
	);
	add_settings_field(
	    'bw_phplivepacker_versionconstant', 
	    'phpLivePacker version constant', 
	    'bw_phplivepacker_createOptionsField_versionConstant', 
	    'bwphplivepacker-settingspage',
	    'bwphplivepacker-defaultsection'			
	);
}


// output additional info for settings setting if necessary
function bw_phplivepacker_sectionInfo_defaultsection(){
	
}


// output fields
function bw_phplivepacker_createOptionsField_cacheDirectory(){
       ?>
       <input type="text" id="bw_phplivepacker_cachedirectory" name="bw_phplivepacker_cachedirectory" value="<?php echo get_option('bw_phplivepacker_cachedirectory'); ?>" size="75"/>
       <p class="description">path from Wordpress root to phpLivePacker cache directory (e.g. wp-content/themes/bartlweb/phplivepacker/cache/)</p>
       <?php
}

function bw_phplivepacker_createOptionsField_enableVersionIncrement(){
       ?>
       <input type="checkbox" id="bw_phplivepacker_enableversionincrement" name="bw_phplivepacker_enableversionincrement" value="1" <?php if(get_option('bw_phplivepacker_enableversionincrement')){ echo 'checked="checked"';} ?>/>
       <p class="description">enables the automatic increment of the version constant on clear phplivepacker cache by modifying the phplivepacker.php php file</p>
       <?php
}

function bw_phplivepacker_createOptionsField_phpFile(){
       ?>
       <input type="text" id="bw_phplivepacker_phpfile" name="bw_phplivepacker_phpfile" value="<?php echo get_option('bw_phplivepacker_phpfile'); ?>" size="75"/>
       <p class="description">path from Wordpress root folder to phplivepacker.php file (e.g. wp-content/themes/bartlweb/phplivepacker.php)</p>
       <?php
}

function bw_phplivepacker_createOptionsField_versionConstant(){
       ?>
       <input type="text" id="bw_phplivepacker_versionconstant" name="bw_phplivepacker_versionconstant" value="<?php echo get_option('bw_phplivepacker_versionconstant'); ?>" size="75"/>
       <p class="description">name of PHP constant used to store the actual phpLivePacker css/js-version count (e.g. PHPLIVEPACKER_VERSION)</p>
       <?php
}

// create options page
function bw_phplivepacker_options_page() {
?>

<div class="wrap">
	<?php screen_icon(); ?>
	<h2>phpLivePacker Settings</h2>
	
	<form action="options.php" method="post">
		 <?php
		    settings_fields('bw_phplivepacker');	
		    do_settings_sections('bwphplivepacker-settingspage');
		?>
		<?php submit_button(); ?>
	</form>
</div>
 
<?php
}


?>